# @summary Manage strongswan service via vici/swanctl.
#
# Besides ensuring that strongswan-swantcl is installed active and the stroke-based
# service is disabled, this module writes strongswan.conf and swanctl.conf. If route based
# is enabled, xfrm interfaces are also managed.
#
# @param aikgen aikgen block in strongswan.conf.
# @param attest attest block in strongswan.conf.
# @param charon charon block in strongswan.conf.
# @param charon_mm charon-mm block in strongswan.conf.
# @param charon_systemd charon-systemd block in strongswan.conf.
# @param imv_policy_manager imv_policy_manager block in strongswan.conf.
# @param libimcs libimcs block in strongswan.conf.
# @param libtls libtls block in strongswan.conf.
# @param libtnccs libtnccs block in strongswan.conf.
# @param manager manager block in strongswan.conf.
# @param medcli medcli block in strongswan.conf.
# @param medsrv medsrv block in strongswan.conf.
# @param pki pki block in strongswan.conf.
# @param pool pool block in strongswan.conf.
# @param pt_tls_client pt-tls-client block in strongswan.conf.
# @param sec_updater sec-updater block in strongswan.conf.
# @param sw_collector sw-collector block in strongswan.conf.
# @param starter starter block in strongswan.conf.
# @param swanctl swanctl block in strongswan.conf.
# @param authorities authorities block in swanctl.conf.
# @param connections connections block in swanctl.conf. Besides keys allowed in swanctl, these ones are added:
#      * base_interface: base interface for xfrm device.
#      * if_id: interface ID for xfrm device used for in and out. In swanctl.conf, this value is used for id_id_in and if_id_out.
# @param secrets secrets block in swanctl.conf.
# @param pools pools block in swanctl.conf.
# @param route_based Whether route based VPN is configured.
# @param xfrm_default_base_interface XRFM base interface used if none is specified in connection.
#
# @example
#   include strongswan
class strongswan (
  Hash $aikgen = {},
  Hash $attest = {},
  Hash $charon = {},
  Hash $charon_mm = {},
  Hash $charon_systemd = {},
  Hash $imv_policy_manager = {},
  Hash $libimcs = {},
  Hash $libtls = {},
  Hash $libtnccs = {},
  Hash $manager = {},
  Hash $medcli = {},
  Hash $medsrv = {},
  Hash $pki = {},
  Hash $pool = {},
  Hash $pt_tls_client = {},
  Hash $sec_updater = {},
  Hash $sw_collector = {},
  Hash $starter = {},
  Hash $swanctl = {},
  Hash $authorities = {},
  Hash $connections = {},
  Hash $secrets = {},
  Hash $pools = {},
  Boolean $route_based = false,
  String $xfrm_default_base_interface = 'lo',
) {
  ensure_packages(['strongswan-swanctl', 'charon-systemd'])
  service { ['ipsec,', 'strongswan-starter']:
    ensure => stopped,
    enable => false,
  }
  service { 'strongswan':
    ensure => running,
    enable => true,
  }
  if ($route_based) {
    $d = strongswan::extract_xfrm_devices_from_connections($connections, $xfrm_default_base_interface)
    $_connections = $d['connections']
    ensure_resources('strongswan::xfrm', $d['xfrm'])
  } else {
    $_connections = $connections
  }
  file { '/etc/strongswan.conf':
    owner   => 'root',
    mode    => '0600',
    content => strongswan::hash_to_strongswan_config({
        aikgen             => $aikgen,
        attest             => $attest,
        charon             => $charon,
        charon-mm          => $charon_mm,
        charon-systemd     => $charon_systemd,
        imv_policy_manager => $imv_policy_manager,
        libimcs            => $libimcs,
        libtls             => $libtls,
        libtnccs           => $libtnccs,
        manager            => $manager,
        medcli             => $medcli,
        medsrv             => $medsrv,
        pki                => $pki,
        pool               => $pool,
        pt-tls-client      => $pt_tls_client,
        sec-updater        => $sec_updater,
        sw-collector       => $sw_collector,
        starter            => $starter,
        swanctl            => $swanctl,
    }),
    notify  => Service['strongswan'],
  }
  file { '/etc/swanctl/swanctl.conf':
    owner   => 'root',
    mode    => '0600',
    content => strongswan::hash_to_strongswan_config({
        authorities => $authorities,
        connections => $_connections,
        secrets     => $secrets,
        pools       => $pools,
    }),
    notify  => Service['strongswan'],
  }
}
